using System.Runtime.InteropServices;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
// The following sets up and runs a multi-channel scan that scans:
// main chan 0, sub chan 0
// main chan 1, sub chan 0,1,2
// main chan 2, sub chan 2,3,4
// Note that if no multiplexer is connected and a sub channel is read, the sub-channel
//   result that is returned will be the same as the main channel - in other words, you
//   can safely experiment with the code by setting up as many channel and sub-channel
//   groups as you'd like without creating a problem. The only thing that requires attention
//   when doing that, is that you allocate enough memory in the data array when making the
//   call to EX_GetScanDataDbl(...). Keep in mind that one scan is comprised of all the 
//   channels and sub-channels. In other words, if your channel setup call looks like:
//   EX_SetMultiChanScanChanCodes(15,15,15,15,15,15,0) then you need to do:
//      double[] dataBuff = new double[96]; //(6 main chans) * (16 sub chans) = 96
//      EX_GetScanDataDbl(dataBuff, 1, numScansReturned, numScansInBuff, scanRunning); 
//    afterwhich, if scanRunning==1 and numScansReturned==1 then the first 16 bytes in
//    dataBuff, will be for main-0, sub0-15, followed by main-1, sub0-15 
// Data in this sample app is read by pressing a button.  For continuous read, 
//  just put data-read function into thread or timer event
namespace CSharpTest
{
    public partial class Form1 : Form
    {
        [DllImport("M201_SP.dll")] public static extern byte EX_SetCOMport(byte bPort);
        [DllImport("M201_SP.dll")] public static extern byte EX_SetScanType(byte bScanType);
        [DllImport("M201_SP.dll")] public static extern byte EX_SetDataLogOptions(uint uiDataLogMethod,
            uint uiLogFileMaxSize, uint uiMisc);
        [DllImport("M201_SP.dll")] public static extern byte EX_GetScanDataDbl(double[] pdblDataBuff, 
            uint uiNumScansToReturn, uint[] puiNumScansReturned, uint[] puiNumScansInBuff, byte[] pfStillScanning);
        [DllImport("M201_SP.dll")] public static extern byte EX_Run();
        [DllImport("M201_SP.dll")] public static extern byte EX_Stop();
        [DllImport("M201_SP.dll")] public static extern byte EX_SetMultiChanScanChanCodes(byte bChan0, 
            byte bChan1, byte bChan2, byte bChan3, byte bChan4, byte bChan5, byte fUpdateIni);
        public Form1() { InitializeComponent(); }
        private void btnSetCOM_Click(object sender, EventArgs e)
        {
            Byte bRetVal; Byte bPort = (byte)byte.Parse(txtPort.Text);
            bRetVal = EX_SetCOMport(bPort);
        }
        private void Form1_Load(object sender, EventArgs e) { }
        private void button1_Click(object sender, EventArgs e)
        {
            label10.Visible = true;
            textBox7.Text = "please wait - -  -   -   -"; Update();
            byte bRetVal = 0;
            bRetVal = EX_SetScanType(1);
            if (bRetVal == 0) { textBox7.Text = "failed EX_SetScanType"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_SetScanType,  wait ---"; Update();
            bRetVal = EX_SetDataLogOptions(7, 0, 0);
            if (bRetVal == 0)
            { textBox7.Text = "failed EX_SetDataLogOptions"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_SetDataLogOptions,  wait ---"; Update();
            //note that I've used the Hex value of 0x24 for channel 2 subs 2,3,4 rather than decimal 36
            //  since it better illustrates the actual sub channels. The upper nibble of the byte is
            //  the low channel and the lower nibble is the high channel. See "QuickRef.html" for more
            //  information:
            //  http://lawsonlabs.com/Download/Manuals-Drivers/Model201/M201_SP/QuickRef.htm
            //  or for overview: http://lawsonlabs.com/Download/Manuals-Drivers/Model201/M201_SP/
            //Remember that any channel set to 16 will not be included in the scan.
            bRetVal = EX_SetMultiChanScanChanCodes(0, 2, 0x24, 16, 16, 16, 0);
            if (bRetVal == 0)
            { textBox7.Text = "failed EX_SetMultiChanScanChanCodes"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_SetMultiChanScanChanCodes"; label10.Visible = false;
            button2.Enabled = true;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            label10.Visible = true;
            textBox7.Text = "please wait - -  -   -   -"; Update();
            byte bRetVal = 0;
            bRetVal = EX_Run();
            if (bRetVal == 0)
            { textBox7.Text = "failed EX_Run"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_Run"; label10.Visible = false;
            button1.Enabled = false;
            button2.Enabled = false;
            button3.Enabled = true;
            button4.Enabled = true;
            lblScanDataInfo.Visible = true;
        }
        private void button3_Click(object sender, EventArgs e)
        {
            label10.Visible = true;
            textBox7.Text = "please wait - -  -   -   -"; Update();
            byte bRetVal = 0;
            double[] dataBuff = new double[7];
            uint[] numScansReturned = new uint[1]; numScansReturned[0] = 0;
            uint[] numScansInBuff = new uint[1];
            byte[] scanRunning = new byte[1]; scanRunning[0] = 0;
            bRetVal = EX_GetScanDataDbl(dataBuff, 1, numScansReturned, numScansInBuff, scanRunning);
            if (bRetVal == 0)
            { textBox7.Text = "failed EX_GetScanDataDbl"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_GetScanDataDbl";
            textBox10.Text = string.Format("{0}", numScansReturned[0]);
            textBox11.Text = string.Format("{0}", numScansInBuff[0]);
            if (scanRunning[0] == 1) textBox12.Text = "yes";
            else textBox12.Text = "no";
            if (numScansReturned[0] == 1)
            {
                txt_0_0.Text = string.Format("{0:f6}", dataBuff[0]);
                txt_1_0.Text = string.Format("{0:f6}", dataBuff[1]);
                txt_1_1.Text = string.Format("{0:f6}", dataBuff[2]);
                txt_1_2.Text = string.Format("{0:f6}", dataBuff[3]);
                txt_2_2.Text = string.Format("{0:f6}", dataBuff[4]);
                txt_2_3.Text = string.Format("{0:f6}", dataBuff[5]);
                txt_2_4.Text = string.Format("{0:f6}", dataBuff[6]);
            } label10.Visible = false;
        }
        private void button4_Click(object sender, EventArgs e)
        {
            lblScanDataInfo.Visible = false;
            label10.Visible = true;
            textBox7.Text = "please wait - -  -   -   -"; Update();
            byte bRetVal = 0;
            bRetVal = EX_Stop();
            if (bRetVal == 0)
            { textBox7.Text = "failed EX_Stop"; label10.Visible = false; return; }
            textBox7.Text = "succeeded EX_Stop"; label10.Visible = false;
            textBox12.Text = "no";
            button1.Enabled = true;
            button2.Enabled = true;
            button3.Enabled = false;
            button4.Enabled = false;
        }
    }
}




